# NAME

ldap-server-test - Run client-side tests against Stanford OpenLDAP servers

# USAGE

ldap-server-test --test=_test_ --server=_server-list_ \[--master=_ldap-master_\]
                 \[other options\]

# DESCRIPTION

**ldap-server-test** runs the test **test** against the OpenLDAP servers
listed in the comma-delimited list **server-list**. See the section TESTS
below for accepted values of &lt;test>.

The **--master** parameter is only relevant when running the "replication"
test.

# REQUIRED ARGUMENTS

You need to supply the **--test** and **--server** options.

# AUTHENTICATION

The script will use your existing Kerberos credentials, but if you want to
use a different ticket cache, or use simple bind, specify either
(**-t**|**--ticket-cache**) or simple bind (**--simple**).

Whichever credentials use for all the "read-only" tests these credentials
must have read access to these attributes in the accounts tree:

     suKrb4Name
     suNameLF
     suEmailStatus
     suName
     suEmailStatus
     uid
     owner

See the replication and sync-sugal tests below for the credentials
necessary for those tests.

Some of the tests are less thorough when run using **--simple**.

## GSSAPI

When using your own Kerberos credentials you do not need to specify a
ticket cache:

    ldap-server-test --test uids --server ldap-dev1

If you want to use a specific ticket cache file use the **--ticket**
option:

    ldap-server-test --test uids --server ldap-dev1 --ticket /tmp/krb5cc_52777_fqGCGlixrQ

## Simple Binds

For simple binds use the **--simple** options:

    ldap-server-test -test uids --server ldap-dev1 --simple

The simple bind credentials are built into the script. The service
principal used when simple binding is **service/ldap-simplebind-testing**
whose DN is

    cn=ldap-simplebind-testing,cn=service,cn=applications,dc=stanford,dc=edu

The password for the simple bind is read automatically from the wallet
object **password/its-idg/ldap/ldap-simplebind-testing**. So, to
successfully run any tests using the **--simple** option your own kerberos
credentials must have access to that wallet object.
To see which attributes this simple-bind principal can access:

    remctl ldap ldap acl service/ldap-simplebind-testing --expand

If wallet is NOT available, you can use the **--simple-bind-password-file**
option to point to a local file containing the simple-bind password.

# TESTS

## ALL

Run all the tests except for **replication**.

## base

The **base** test does a query against the base DN and does not descend
into any children. This test is very quick and is useful principally to
verify that the OpenLDAP service is running.
Test name sent to Nagios: `ldap_base`.

## quick

The **quick** test attempts to read the "suName" attribute from 100 accounts
tree entries. It is usually quite fast.
Test name sent to Nagios: `ldap_quick`.

## uids

The **uids** test attempts to the "uid" attribute from the accounts
tree. It reads 450 entries when using simple bind and 3000 entries when
binding via GSSAPI. It then chooses a random 10 of these uids and does
some additional reads for those 10 accounts entries.  Test name sent to
Nagios: `ldap_uids`.

As this test reads many accounts entries it can be slow.

## replication-delay

The **replication-delay** test checks to see how far behind a replica is
from a master; as such, the **--master** argument is required for this
test.

The **--sync-delay** should also be provided. This option specifies the
maximum number of seconds the replica can be behind the master before the
replica is considered stale. If the **--sync-delay** is NOT provided, a
default of 10 seconds is assumed.
Test name sent to Nagios: `ldap_replication_delay_M` where `M` is the
last digit of the master's name.

## replication

The **replication** test is special. Unlike the other tests, it both reads
and writes. In particular, it writes to the master and then ensures that
the data is replicated to the supplied replica servers.

As this test writes to the master its use requires the use of the
**--master** option. Here is how to test that writes to the DEV master
server are replicated to two of its replicas:

    ldap-server-test --test replication --master ldap-dev0 \
    --server ldap-dev1,ldap-dev2

Although this test writes to the master server, it writes _only_ to the
**cn=test,dc=stanford,dc=edu** tree, so it is very low-impact.

If you want to test replication to a simple-bind server, then use the
**--simple** option; when doing this be sure that you list only simple-bind
servers in the **--server** option

    ldap-server-test --test replication --master directory-master0 \
    --server directory5,directory6

Note: the **replication** test \*always\* requires Kerberos credentials
as the permissions to write to the master is only granted via GSSAPI.

For the **replication** test to run properly, the credentials that _write_ to
the master must have write access to the
**cn=test,dc=stanford,dc=edu** tree. The credentials that _read_ must have read
access to the **cn=test,dc=stanford,dc=edu** tree. In particular, if
using the **--simple** option, the simple-bind credentials must have read access to
the test tree.
Test name sent to Nagios: `ldap_replication_master_M` where `M` is the
last digit of the master's name.

## starttls

The **starttls** test will ensure that the server can start a TLS session
after GSSAPI authentication and that it sends the correct certificate
chain. Note that this means for simple-bind server that are normally
accessed using username and password rather than GSSAPI this test will
usually fail.

## whois-anonymous

This test will do an anonynmous WHOIS query against the server indicated
by the **--server** option using the string given by the **--whois-query**
option. It collects the result and looks for the string given by the
**--whois-expected** option. Keep in mind that only LDAP replicas run the
WHOIS service. This WHOIS query is anonymous, meaning that it respsects
LDAP visibility settings.

Example:

    ldap-server-test --test whois-anonymous --server ldap-dev2 \
    --whois-query www.stanford.edu --whois-expected 'Office of Public Affairs'
    # This is equivalent to
    whois -h ldap-dev2.stanford.edu www.stanford.edu | grep 'Office of Public Affairs' | grep -Eq '.'

## whois-authenticated

This test is identical to the whois-anonymous test except it uses GSSAPI
credntials to make a remctl call to the specified WHOIS server to get the
WHOIS information.

    ldap-server-test --test whois-authenticated --server ldap-dev2 \
    --whois-query www.stanford.edu --whois-expected 'Office of Public Affairs'
    # This is equivalent to
    remctl ldap-dev2.stanford.edu whoisd query www.stanford.edu | grep 'Office of Public Affairs' | grep -Eq '.'

# Synchronization tests

There are several synchronization scripts that run as listeners or batch
files (or both) on the master servers.  So, we have special
synchronization process tests that can be run against the masters servers.
See the ikiwiki page
https://ikiwiki.stanford.edu/service/ldap/sync-scripts/ for more
information on these synchronization scripts.

WARNING! While the synchronization scripts should leave the data in the
state as it was before the test was run, these scripts do change data in the
the non-test trees. So, running a synchronization script against the
production master can damage the data. Read the instructions for each
test on how to run them so that the risk to the data is minimized.

## sync-sugal

Usage: use the `-s|--server` option to point to the master you are testing
against. Example:

    ldap-server-test --server ldap-master --test sync-sugal --uid whm

This test exercises the ldap-sync-sugal-attributes listener. The test will
change (and then revert the change) these attributes in the people tree:
cn, suothername, and sudisplaynamelf. You specify a single record to test
against by giving the `--uid` option.

**RECOMMENDATION** When running (especially against the production master),
use the uid of someone who has **left Stanford**, so that if the test fails
and you have to clean things up, you are only affecting an unused record.

Credentials: For the **sync-sugal** test to run properly, the credentials
used must have both read and write access to these attributes in the
people tree: cn, suOtherName, suDisplayNameLF, suGALcn, suGALsuOtherName,
suGALsuDisplayNameLF, and suVisibIdentity.

## sync-privgroups

This test exercises the (badly named) ldap-sync-attribute listener. The
test will add and remove a test suPrivilegeGroup to the provided uid's
people record and verify that the suPrivilegeGroup attribute is propagated
to the accounts tree. You specify a single record to test against by
giving the `--uid` option.

**RECOMMENDATION** When running against the production master, use your own
uid so that if the test fails and you have to clean things up, you are
only affecting your own record.

Example:

    ldap-server-test --test sync-privgroups --server ldap-dev0 \
    --uid adamhl

## sync-group-maint

This test exercises the ldap-group-maint listener. The test will add users
to and remove users from a specified user's people tree entry and verify
that those users get added to and removed from that group's groups tree
entry. You must supply both a user's uid and a workgroup.

Example:

    ldap-server-test --test sync-group-maint --server ldap-dev0 \
    --uid adamhl --workgroup acs:adam-test1

**RECOMMENDATION** When running against the production master, use a test
workgroup so that if the test fails and you have to clean things up, you are
only affecting an unimportant workgroup.

## sync-wg-maint

This test will change a user's `suSeasLocal` attribute in the accounts
tree and verify that the ldap-wg-maint sync process changes the user's
itservices:\* mail group membership. The test then changes the
`suSeasLocal` attribute back to its pre-test value. Be careful when
running this in production as the changes it makes affect mail delivery;
in other words, if the test fails to run properly it will BREAK mail
dilvery for the the person being tested against.

Example:

    ldap-server-test --test sync-wg-maint --server ldap-dev0 \
    --uid adamhl

**RECOMMENDATION** Do not run this in production until we have a test
account whose e-mail we can afford to break.

# OPTIONS

- **-s|--server**

    A comma-delimited list of servers to run tests against. This is required
    for all tests. The server name can be followed by ":port" to indicate a port
    other than 389. For example, to connect using port 989:

        ldap-server-test --test uids --server ldap-dev1.stanford.edu:989

- **--test**

    A required option specifying which test to run. See the above section
    **TESTS** for a description of which tests are available.

- **--simple**

    Use simple bind when reading the OpenLDAP. See **AUTHENTICATION** for more
    information on simple binds.

- **--simple-bind-password-file** _file\_containing\_password_

    By default, when using the **--simple** option the simple-bind password
    is read from a wallet object. However, there are circumstances where
    you might want to read the password from a local file. If so, use
    **--simple-bind-password-file**.

- **-t|--ticket-cache** _ticket\_cache\_file_

    Use _ticket\_cache\_file_ as GSSAPI credential source when reading and (in
    the case of the **replication** test) writing.

- **--master**

    Specify which server is master for the **replication** test.

- **--delay** _seconds_

    When running the replication test, add this many extra seconds between
    checks that the data has replicated. This is useful for slow servers or
    slow connections.

- **--time-offset** _seconds_,_seconds_

    If this option is given, it should be given as a comma separated pair of
    two integers `m,n`. The script will wait a random number of seconds
    before running, waiting at least `m` seconds and no more than `n`
    seconds. For example, the option `--time-offset 300,600` means wait
    a random number of seconds between 300 seconds and 600 seconds. This is
    useful if you want to stagger your tests so they don't all happen at the
    same time.

- **--nagios-servers**

    Runs tests in Nagios-compatible mode and send results to each
    Nagios server whose IP is listed in the comma-delimited
    list **--nagios-servers**.

- **--test-name-override** _test-name_

    When Nagios servers are indicated using the **--nagios-servers** option
    messages sent to the Nagios server will contain the test result, the
    hostname, and the name of the test. For example, if you run the
    **uids** test the test name sent is **ldap\_uids**. If you need to override
    the test name, use **--test-name-override**. You cannot use this
    option with the **ALL** set of tests. Example

        ldap-server-test --nagios-servers=1.2.3.4 --test uids \
          --server ldap-dev1 --test-name-override=custom-uids

- **--protocol**

    By default, the **ldap://** protocol is used for GSSAPI binds and
    **ldaps://** is used for simple binds. However, if you want to OVERRIDE
    this (e.g., to do a GSSAPI bind against **ldaps://**) you can specify this
    using the **--protocol** option. Example:

        ldap-server-test --test uids --server ldap-dev1 --protocol ldaps://

- **--no-start-tls**

    When authenticating using the ldap:// protocol the bind, by default, makes
    a StartTLS request. If you need to authenticate using ldap:// but do **not**
    want TLS to start, use the **--no-start-tls** option.  Note that this
    option is ignored if the protocol is set, either implicitly or explicitly,
    to ldaps://.

- **--whois-query**

    This option is only relevant with the WHOIS tests. When running one of the
    WHOIS tests it is required and is used as the WHOIS query. See the WHOIS
    test above for examples of its use.

- **--whois-expected**

    This option is only relevant with the WHOIS tests. When running one of the
    WHOIS tests it is required and is to verify that the WHOIS query worked
    properly: if the argument to **--whois-expected** is detected in the
    resulting WHOIS qiery, then the test passes, otherwise the test
    failes. This string should be a PCRE. See the WHOIS test above for
    examples of its use.

# EXIT STATUS

The script will exit with 0 if the script completes and there were no
failures, 1 for any other reason.

# NOTES

Later.

# EXAMPLES

Run all tests (except **replication**):

    ldap-server-test --test ALL --server ldap-dev1

Run all tests against two different servers:

    ldap-server-test --test ALL --server ldap-dev1,ldap-test2

Test against the base DN:

    ldap-server-test --test base --server ldap-dev1

Run the "quick" read test:

    ldap-server-test --test quick --server ldap-dev1

Run the deeper "uids" read test:

    ldap-server-test --test uids --server ldap-dev1

Run the replication test against the DEV master and two of the replicas:

    ldap-server-test --test replication --server ldap-dev1,ldap-dev2 --master ldap-dev0

# DEPENDENCIES

[Data::Dumper](https://metacpan.org/pod/Data::Dumper),
[Pod::Usage](https://metacpan.org/pod/Pod::Usage),
[Time::HiRes](https://metacpan.org/pod/Time::HiRes),
[English](https://metacpan.org/pod/English),
[Test::More](https://metacpan.org/pod/Test::More),
[Carp](https://metacpan.org/pod/Carp),
[Clone](https://metacpan.org/pod/Clone),
[Getopt::Long::Descriptive](https://metacpan.org/pod/Getopt::Long::Descriptive),
[Cache::FileCache](https://metacpan.org/pod/Cache::FileCache),
[Stanford::LDAP::RegressionTesting](https://metacpan.org/pod/Stanford::LDAP::RegressionTesting),
[Stanford::Orange::Util](https://metacpan.org/pod/Stanford::Orange::Util),
[Stanford::Orange::Nagios](https://metacpan.org/pod/Stanford::Orange::Nagios),
[Stanford::FlexTest::Nagios](https://metacpan.org/pod/Stanford::FlexTest::Nagios)

# INCOMPATIBILITIES

No known incompatibilities.

# BUGS AND LIMITATIONS

None known.

# DIAGNOSTICS

More later.

# CONFIGURATION

Configuration is managed via the options.

# SEE ALSO

ldapsearch(1)

# AUTHOR

Adam Lewenberg <adamhl@stanford.edu>

# LICENSE AND COPYRIGHT

Copyright 2016,2019 The Board of Trustees of the Leland Stanford Junior
University.  All rights reserved.

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided
that the above copyright notice appear in all copies and that both that
copyright notice and this permission notice appear in supporting
documentation, and that the name of Stanford University not be used in
advertising or publicity pertaining to distribution of the software
without specific, written prior permission.  Stanford University makes no
representations about the suitability of this software for any purpose.
It is provided "as is" without express or implied warranty.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
